use once_cell::sync::OnceCell;
use paris::info;
use std::{sync::Arc, time::Duration};
use tokio::{sync::Mutex, time::sleep};
use tracing::warn;

use crate::{picker::led_color::LedColor, rpc};

pub struct CoreManager {
    flowing_light_mode: Arc<Mutex<bool>>,
}

impl CoreManager {
    pub fn global() -> &'static CoreManager {
        static CORE_MANAGER: OnceCell<CoreManager> = OnceCell::new();

        CORE_MANAGER.get_or_init(|| CoreManager {
            flowing_light_mode: Arc::new(Mutex::new(false)),
        })
    }

    pub async fn play_flowing_light(&self) {
        *self.flowing_light_mode.lock().await = true;

        let mut hue = 0f64;
        let step_length = 2.0;
        while self.flowing_light_mode.lock().await.to_owned() {
            let mut colors = Vec::<LedColor>::new();
            for i in 0..60 {
                let color = LedColor::from_hsv((hue + i as f64 * step_length) % 360.0, 1.0, 0.5);
                colors.push(color);
            }
            hue = (hue + 1.0) % 360.0;
            match rpc::manager::Manager::global()
                .publish_led_colors(&colors)
                .await
            {
                Ok(_) => {}
                Err(error) => {
                    warn!("publish led colors failed. {}", error);
                }
            }
            sleep(Duration::from_millis(10)).await;
        }
    }

    pub async fn stop_flowing_light(&self) {
        let mut mode = self.flowing_light_mode.lock().await;
        *mode = false;
    }
}
